Package: GOaGO
Title: Gene Ontology enrichment analysis of gene pairs
Version: 1.0.0
Authors@R: person("Aleksander", "Jankowski",
    email = "aleksander.jankowski@uw.edu.pl",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-2212-6224"))
Description: GO-a-GO annotates Gene Ontology terms that are enriched in a given
    set of gene pairs. The enrichment is calculated from a permutation test for
    overrepresentation of gene pairs that are associated with a shared term.
    Such gene pairs are counted for the original set of gene pairs and compared
    against randomized sets in which the structure of the pairs is preserved,
    but the gene identities (including the associated terms) are permuted.
License: Artistic-2.0
Encoding: UTF-8
LazyData: false
Imports: AnnotationDbi, BiocGenerics, BiocParallel, clusterProfiler,
        data.table, DOSE, GenomeInfoDb, GenomicRanges, ggplot2,
        ggridges, Matrix, qvalue, S4Vectors
Depends: R (>= 4.4.0), methods
biocViews: GO, GeneSetEnrichment
URL: https://github.com/ajank/GOaGO
BugReports: https://github.com/ajank/GOaGO/issues
RoxygenNote: 7.3.3
Suggests: BiocStyle, GenomicInteractions, ggrepel, knitr, org.Hs.eg.db,
        rmarkdown, rtracklayer, testthat,
        TxDb.Hsapiens.UCSC.hg19.knownGene
Config/testthat/edition: 3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GOaGO
git_branch: RELEASE_3_23
git_last_commit: ace0eb8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:33:02 UTC; biocbuild
Author: Aleksander Jankowski [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2212-6224>)
Maintainer: Aleksander Jankowski <aleksander.jankowski@uw.edu.pl>
