% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R, R/methods-show.R
\name{GOaGO-accessors}
\alias{GOaGO-accessors}
\alias{genePairs}
\alias{keyType}
\alias{organism}
\alias{organism,GOaGO-result-method}
\alias{show}
\alias{show,GOaGO-result-method}
\title{Accessors and show method for \code{GOaGO-result} objects}
\usage{
genePairs(object)

keyType(object)

organism(object)

show(object)
}
\arguments{
\item{object}{of class \code{GOaGO-result}}
}
\value{
\code{genePairs} returns a data frame with the input gene pairs, with the
columns \code{geneID1}, \code{geneID2} and \code{pairID}.

\code{keyType} returns the type of gene identifiers, such as "ENTREZID" or
"ENSEMBL".

\code{organism} returns the scientific name (i.e. genus and species, or
genus and species and subspecies) of the organism.

\code{show} displays the object, and returns an invisible \code{NULL}.
}
\description{
Accessors and show method for \code{GOaGO-result} objects
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
show(goago)

genePairs(goago)
keyType(goago)
organism(goago)
}
