% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\name{GOaGO}
\alias{GOaGO}
\title{Gene Ontology enrichment analysis in a set of gene pairs}
\usage{
GOaGO(
  genePairs,
  OrgDb,
  keyType = NULL,
  ont = "MF",
  minCount = 1,
  numPermutations = 10000,
  universe,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  qvalueCutoff = 0.2,
  minGSSize = 10,
  maxGSSize = 500
)
}
\arguments{
\item{genePairs}{a data frame with columns \code{geneID1} and \code{geneID2}
containing gene identifiers; column \code{pairID} will also be used if
provided.}

\item{OrgDb}{OrgDb}

\item{keyType}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL", if
it cannot be determined from metadata of \code{genePairs}}

\item{ont}{one of "BP", "MF", and "CC" subontologies, or "ALL" for all three}

\item{minCount}{cutoff for number of pairs that share a GO term for this
term to be considered}

\item{numPermutations}{number of permutations performed in the enrichment
test}

\item{universe}{a set of background genes. If missing, all the genes from
all the gene pairs will be used as background.}

\item{pvalueCutoff}{adjusted p-value cutoff on enrichment tests to report}

\item{pAdjustMethod}{one of "holm", "hochberg", "hommel", "bonferroni",
"BH", "BY", "fdr", "none"}

\item{qvalueCutoff}{q-value cutoff on enrichment tests to report as
significant. Tests must pass (i) \code{pvalueCutoff} on unadjusted
p-values, (ii) \code{pvalueCutoff} on adjusted p-values, and (iii)
\code{qvalueCutoff} on q-values to be reported.}

\item{minGSSize}{minimal size of genes annotated for testing}

\item{maxGSSize}{maximal size of genes annotated for testing}
}
\value{
A \code{GOaGO-result} object.
}
\description{
Given a data frame of gene pairs, this function will return the enriched
Gene Ontology terms after FDR control.
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
show(goago)
}
\seealso{
\code{\link{GOaGO-result-class}}
}
