% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\docType{class}
\name{GOaGO-result-class}
\alias{GOaGO-result-class}
\title{An S4 class to represent the results of GO-a-GO enrichment analysis}
\description{
An S4 class to represent the results of GO-a-GO enrichment analysis
}
\section{Slots}{

\describe{
\item{\code{result}}{A data frame of the enriched Gene Ontology terms, with the
following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
the GO term), \code{Count} (number of input gene pairs sharing the given
term), \code{PairRatio} (fraction of input gene pairs sharing the given
term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
term), \code{FoldEnrichment} (quotient of the two fractions),
\code{pvalue}, \code{p.adjust}, \code{qvalue}.}

\item{\code{pvalueCutoff}}{adjusted p-value cutoff on enrichment tests}

\item{\code{pAdjustMethod}}{p-value adjustment method}

\item{\code{qvalueCutoff}}{q-value cutoff on enrichment tests}

\item{\code{minCount}}{cutoff for number of pairs that share a GO term for this term
to be considered}

\item{\code{numPermutations}}{number of permutations performed in the enrichment
test}

\item{\code{minGSSize}}{minimal size of genes annotated for testing}

\item{\code{maxGSSize}}{maximal size of genes annotated for testing}

\item{\code{organism}}{scientific name of the organism}

\item{\code{ontology}}{one of "BP", "MF", and "CC" subontologies, or "ALL" for all
three}

\item{\code{keyType}}{type of gene identifiers, such as "ENTREZID" or "ENSEMBL"}

\item{\code{genePairs}}{A data frame with the input gene pairs, with the columns
\code{geneID1}, \code{geneID2} and \code{pairID}.}

\item{\code{pairTerms}}{A data frame linking the enriched Gene Ontology terms with
the input gene pairs, with the columns \code{pairID} and \code{ID} (of the
GO term).}

\item{\code{permutedResult}}{A data frame with the columns \code{ID} (of the GO
term) and \code{Count}, keeping the numbers of permuted gene pairs sharing
the term as obtained in every random permutation.}

\item{\code{universe}}{a set of background genes}
}}

