% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO-package.R
\docType{package}
\name{GOaGO-package}
\alias{GOaGO-package}
\title{GOaGO: Gene Ontology enrichment analysis of gene pairs}
\description{
GO-a-GO annotates Gene Ontology terms that are enriched in a given set of gene pairs. The enrichment is calculated from a permutation test for overrepresentation of gene pairs that are associated with a shared term. Such gene pairs are counted for the original set of gene pairs and compared against randomized sets in which the structure of the pairs is preserved, but the gene identities (including the associated terms) are permuted.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ajank/GOaGO}
  \item Report bugs at \url{https://github.com/ajank/GOaGO/issues}
}

}
\author{
\strong{Maintainer}: Aleksander Jankowski \email{aleksander.jankowski@uw.edu.pl} (\href{https://orcid.org/0000-0002-2212-6224}{ORCID})

}
\keyword{internal}
