% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOaGO.R
\name{uniqueGenePairs}
\alias{uniqueGenePairs}
\title{Extract unique gene pairs from the data frame provided}
\usage{
uniqueGenePairs(genePairs)
}
\arguments{
\item{genePairs}{a data frame with columns \code{geneID1} and \code{geneID2}
containing gene identifiers; column \code{pairID} will also be used if
provided.}
}
\value{
A data frame with columns \code{pairID}, \code{geneID1} and
  \code{geneID2}. If loops or duplicates were removed, a warning will
  alert you. If column \code{pairID} was not provided in \code{genePairs},
  an integer vector equal to \code{seq_len(nrow(result))} will be used.
}
\description{
Given a data frame of gene pairs, this function will return the unique
pairs of genes, removing loops (gene pairs containing the same gene twice)
and duplicates. Note that gene pair (A, B) is a duplicate of (B, A).
}
