% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\docType{methods}
\name{as.data.frame}
\alias{as.data.frame}
\alias{as.data.frame,GOaGO-result-method}
\title{Coerce a \code{GOaGO-result} object to a data frame}
\usage{
as.data.frame(x, row.names=NULL, optional=FALSE, ...)
}
\arguments{
\item{x}{The object to coerce.}

\item{row.names, optional, ...}{Not used, inherited from
\code{base::as.data.frame()}.}
}
\value{
A data frame of the enriched Gene Ontology terms, with the
  following columns: \code{ONTOLOGY}, \code{ID}, \code{Description} (all of
  the GO term), \code{Count} (number of input gene pairs sharing the given
  term), \code{PairRatio} (fraction of input gene pairs sharing the given
  term), \code{BgRatio} (fraction of permuted gene pairs sharing the given
  term), \code{FoldEnrichment} (quotient of the two fractions),
  \code{pvalue}, \code{p.adjust}, \code{qvalue}.
}
\description{
Coerce a \code{GOaGO-result} object to a data frame
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
as.data.frame(goago)
}
