% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/termGenePairs.R
\name{termGenePairs}
\alias{termGenePairs}
\title{Extract the enriched Gene Ontology terms along with gene pairs sharing them}
\usage{
termGenePairs(object, OrgDb = NULL)
}
\arguments{
\item{object}{of class \code{GOaGO-result}}

\item{OrgDb}{OrgDb to map gene identifiers to gene symbols}
}
\value{
A data frame similar to returned by \code{as.data.frame(object)},
  but including all the gene pairs sharing each enriched Gene Ontology term,
  one gene pair in each row. Additional columns include \code{pairID},
  \code{geneID1}, \code{geneID2} and any other columns provided in
  \code{genePairs} argument to \code{\link{GOaGO}}. If \code{OrgDb} is
  provided, \code{geneSymbol1} and \code{geneSymbol2} will also be added.
}
\description{
Extract the enriched Gene Ontology terms along with gene pairs sharing them
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
termGenePairs(goago, OrgDb = org.Hs.eg.db)
}
\seealso{
\code{\link{as.data.frame,GOaGO-result-method}}
}
