% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{genePairsGM12878}
\alias{genePairsGM12878}
\title{Gene pairs associated with chromatin loops in GM12878 cell line}
\format{
A data frame with 2,339 rows and 13 columns:
\describe{
  \item{interactionID}{loop identifier}
  \item{chrom1}{chromosome of loop anchor 1}
  \item{start1}{start coordinate of loop anchor 1}
  \item{end1}{end coordinate of loop anchor 1}
  \item{geneID1}{Entrez identifier of the gene associated to loop anchor 1}
  \item{tss1}{TSS coordinate of the associated gene}
  \item{strand1}{strand (\code{"+"} or \code{"-"}) of the associated gene}
  \item{chrom2}{chromosome of loop anchor 2}
  \item{start2}{start coordinate of loop anchor 2}
  \item{end2}{end coordinate of loop anchor 2}
  \item{geneID2}{Entrez identifier of the gene associated to loop anchor 2}
  \item{tss2}{TSS coordinate of the associated gene}
  \item{strand2}{strand (\code{"+"} or \code{"-"}) of the associated gene.}
}
}
\source{
https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE63525
}
\usage{
genePairsGM12878
}
\description{
The dataset is based on 9,448 chromatin loops identified in human cell line
GM12878 as peaks in Hi-C contact maps. Of these chromatin loops, 1,581
overlapped at least one gene Transcription Start Site (TSS) at both loop
anchors. As some loop anchors overlapped multiple TSSes, possibly of
different genes, the dataset contains all combinations for these loops,
yielding a total of 2,339 gene pairs, of which 1,743 pairs are unique and do
not contain the same gene twice.
}
\references{
Rao, S. S., Huntley, M. H., \emph{et al.} (2014). A 3D map of
  the human genome at kilobase resolution reveals principles of chromatin
  looping. \emph{Cell}, 159(7), 1665-80.
}
\keyword{datasets}
