% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{RidgePlot}
\alias{RidgePlot}
\title{Ridgeplot of the sampling distributions for the randomized gene pairs}
\usage{
RidgePlot(
  object,
  minCount = 5,
  showCategory = 10,
  orderBy = "FoldEnrichment",
  decreasing = TRUE,
  font.size = 12,
  label_format = 50
)
}
\arguments{
\item{object}{GO-a-GO results of class \code{GOaGO-result}}

\item{minCount}{plot only the GO terms that are associated to at least the
given number of gene pairs}

\item{showCategory}{number of terms to display or a vector of terms}

\item{orderBy}{The order of the Y-axis, one of \code{"FoldEnrichment"},
\code{"PairRatio"} and \code{"Count"}.}

\item{decreasing}{logical. Should the \code{orderBy} order be increasing or
decreasing?}

\item{font.size}{font size}

\item{label_format}{a numeric value sets wrap length, alternatively a custom
function to format axis labels. By default wraps names longer than 50
characters.}
}
\value{
A \code{ggplot} object that can be further customized using the
  \code{ggplot2} package.
}
\description{
Ridgeplot of the sampling distributions of numbers of gene pairs sharing
each enriched Gene Ontology term, obtained for the randomized gene pairs.
}
\examples{
library(org.Hs.eg.db)
data("genePairsGM12878")

goago <- GOaGO(genePairsGM12878, keyType = "ENTREZID", OrgDb = org.Hs.eg.db)
RidgePlot(goago)
}
